@@ -26,7 +26,7 @@ from azure.cli.core.extension import get_extension
 from azure.cli.core.util import (
     get_command_type_kwarg, read_file_content, get_arg_list, poller_classes)
 from azure.cli.core.local_context import LocalContextAction
-import azure.cli.core.telemetry as telemetry
+from azure.cli.core import telemetry
 from azure.cli.core.commands.progress import IndeterminateProgressBar
 
 from knack.arguments import CLICommandArgument
@@ -35,7 +35,7 @@ from knack.deprecation import ImplicitDeprecated, resolve_deprecate_info
 from knack.invocation import CommandInvoker
 from knack.preview import ImplicitPreviewItem, PreviewItem, resolve_preview_info
 from knack.experimental import ImplicitExperimentalItem, ExperimentalItem, resolve_experimental_info
-from knack.log import get_logger
+from knack.log import get_logger, CLILogging
 from knack.util import CLIError, CommandResultItem, todict
 from knack.events import EVENT_INVOKER_TRANSFORM_RESULT
 from knack.validators import DefaultStr
@@ -130,6 +130,11 @@ class CacheObject:
             raise CLIError('cloud name unexpectedly empty')
         copy_kwargs = kwargs.copy()
         copy_kwargs.pop('self', None)
+        # handle "content_type" introduced by https://gist.github.com/iscai-msft/47fe6ecbdd06013bb19a2e16f85a0b43
+        # e.g., 2018-03-01-hybrid when execute "az network vnet update"
+        #       https://github.com/Azure/azure-sdk-for-python/pull/24162
+        #       sdk/network/azure-mgmt-network/azure/mgmt/network/v2017_10_01/operations/_operations.py:22897
+        copy_kwargs.pop('content_type', None)
         resource_group = copy_kwargs.pop('resource_group_name', None) or args[0]
 
         if len(args) > 2:
@@ -261,13 +266,14 @@ class AzCliCommand(CLICommand):
 
     def __init__(self, loader, name, handler, description=None, table_transformer=None,
                  arguments_loader=None, description_loader=None,
-                 formatter_class=None, deprecate_info=None, validator=None, **kwargs):
+                 formatter_class=None, sensitive_info=None, deprecate_info=None, validator=None, **kwargs):
         super(AzCliCommand, self).__init__(loader.cli_ctx, name, handler, description=description,
                                            table_transformer=table_transformer, arguments_loader=arguments_loader,
                                            description_loader=description_loader, formatter_class=formatter_class,
                                            deprecate_info=deprecate_info, validator=validator, **kwargs)
         self.loader = loader
         self.command_source = None
+        self.sensitive_info = sensitive_info
         self.no_wait_param = kwargs.get('no_wait_param', None)
         self.supports_no_wait = kwargs.get('supports_no_wait', False)
         self.exception_handler = kwargs.get('exception_handler', None)
@@ -562,7 +568,8 @@ class AzCliCommandInvoker(CommandInvoker):
         self.cli_ctx.raise_event(EVENT_INVOKER_CMD_TBL_LOADED, cmd_tbl=self.commands_loader.command_table,
                                  parser=self.parser)
 
-        arg_check = [a for a in args if a not in ['--debug', '--verbose']]
+        arg_check = [a for a in args if a not in
+                     (CLILogging.DEBUG_FLAG, CLILogging.VERBOSE_FLAG, CLILogging.ONLY_SHOW_ERRORS_FLAG)]
         if not arg_check:
             self.parser.enable_autocomplete()
             subparser = self.parser.subparsers[tuple()]
@@ -668,6 +675,9 @@ class AzCliCommandInvoker(CommandInvoker):
         event_data = {'result': results}
         self.cli_ctx.raise_event(EVENT_INVOKER_FILTER_RESULT, event_data=event_data)
 
+        if not self.data['output'] or self.data['output'] != 'none':
+            self._resolve_output_sensitive_data_warning(cmd, event_data['result'])
+
         # save to local context if it is turned on after command executed successfully
         if self.cli_ctx.local_context.is_on and command and command in self.commands_loader.command_table and \
                 command in self.parser.subparser_map and self.parser.subparser_map[command].specified_arguments:
@@ -704,6 +714,7 @@ class AzCliCommandInvoker(CommandInvoker):
                 result = list(result)
 
             result = todict(result, AzCliCommandInvoker.remove_additional_prop_layer)
+
             event_data = {'result': result}
             cmd_copy.cli_ctx.raise_event(EVENT_INVOKER_TRANSFORM_RESULT, event_data=event_data)
             return event_data['result']
@@ -718,7 +729,7 @@ class AzCliCommandInvoker(CommandInvoker):
             expanded_arg, cmd_copy = job
             try:
                 results.append(self._run_job(expanded_arg, cmd_copy))
-            except(Exception, SystemExit) as ex:  # pylint: disable=broad-except
+            except (Exception, SystemExit) as ex:  # pylint: disable=broad-except
                 exceptions.append((ex, id_arg))
         return results, exceptions
 
@@ -806,6 +817,36 @@ class AzCliCommandInvoker(CommandInvoker):
         if isinstance(cmd.command_source, ExtensionCommandSource) and cmd.command_source.overrides_command:
             logger.warning(cmd.command_source.get_command_warn_msg())
 
+    def _resolve_output_sensitive_data_warning(self, cmd, result):
+        if not cmd.cli_ctx.config.getboolean('clients', 'show_secrets_warning', False):
+            return
+
+        from ..credential_helper import sensitive_data_detailed_warning_message, sensitive_data_warning_message
+        sensitive_info = cmd.sensitive_info if hasattr(cmd, 'sensitive_info') else None
+        if sensitive_info:
+            message = sensitive_data_warning_message
+            if sensitive_info.sensitive_keys:
+                message = sensitive_data_detailed_warning_message.format(', '.join(sensitive_info.sensitive_keys))
+            logger.warning(message)
+            return
+
+        from ..credential_helper import distinguish_credential
+        from ..telemetry import set_secrets_detected
+        try:
+            containing_credential, secret_property_names = distinguish_credential(result)
+            if not containing_credential:
+                set_secrets_detected(False)
+                return
+
+            message = sensitive_data_warning_message
+            if secret_property_names:
+                message = sensitive_data_detailed_warning_message.format(', '.join(secret_property_names))
+            logger.warning(message)
+            set_secrets_detected(True)
+        except Exception:  # pylint: disable=broad-except
+            # ignore all exceptions, as this is just a warning
+            pass
+
     def resolve_confirmation(self, cmd, parsed_args):
         confirm = cmd.confirmation and not parsed_args.__dict__.pop('yes', None) \
             and not cmd.cli_ctx.config.getboolean('core', 'disable_confirm_prompt', fallback=False)
@@ -1139,13 +1180,14 @@ def _is_paged(obj):
             and not isinstance(obj, dict):
         from msrest.paging import Paged
         from azure.core.paging import ItemPaged as AzureCorePaged
-        return isinstance(obj, (AzureCorePaged, Paged))
+        from azure.cli.core.aaz._paging import AAZPaged
+        return isinstance(obj, (AzureCorePaged, Paged, AAZPaged))
     return False
 
 
 def _is_poller(obj):
     # Since loading msrest is expensive, we avoid it until we have to
-    if obj.__class__.__name__ in ['AzureOperationPoller', 'LROPoller']:
+    if obj.__class__.__name__ in ['AzureOperationPoller', 'LROPoller', 'AAZLROPoller']:
         return isinstance(obj, poller_classes())
     return False
 
@@ -1417,6 +1459,11 @@ class AzCommandGroup(CommandGroup):
         if is_experimental:
             merged_kwargs['experimental_info'] = ExperimentalItem(self.command_loader.cli_ctx, object_type='command')
 
+    def sensitive(self, **kwargs):
+        from .sensitive import SensitiveItem
+        kwargs['object_type'] = 'command'
+        return SensitiveItem(self.command_loader.cli_ctx, **kwargs)
+
 
 def register_cache_arguments(cli_ctx):
     from knack import events
